@@ -15,8 +15,8 @@
  *    written permission.
  *
  * ALTERNATIVELY, this product may be distributed under the terms of
- * the GNU General Public License, in which case the provisions of the GPL2 are
- * required INSTEAD OF the above restrictions.  (This clause is
+ * the GNU General Public License, in which case the provisions of the GPL2
+ * are required INSTEAD OF the above restrictions.  (This clause is
  * necessary due to a potential bad interaction between the GPL and
  * the restrictions contained in a BSD-style copyright.)
  *
@@ -39,35 +39,50 @@
 
 /**
  * Common data required for symmetric and AEAD ciphers
+ * @in: Input data (plaintext for encryption, ciphertext for
+ *	decryption) - input
+ * @inlen: Length of in buffer - input
+ * @out: output buffer (ciphertext for encryption, plaintext for
+ *	 decryption) - output
+ * @outlen: Length of output buffer - input / output
+ * @iv: IV - input
+ * @ivlen: Length of IV - input
  */
 struct kcapi_skcipher_data {
-	const unsigned char *in; /** <Input data (plaintext for encryption,
-				  * ciphertext for decryption) -- input */
-	size_t inlen; /** <Length of in buffer -- input */
-	unsigned char *out; /** <output buffer (ciphertext for encryption,
-			     * plaintext for decryption) -- output */
-	size_t outlen; /** <Length of output buffer -- input */
-	const unsigned char *iv; /** <IV -- input */
-	size_t ivlen; /** <Length of IV -- input */
+	const unsigned char *in;
+	size_t inlen;
+	unsigned char *out;
+	size_t outlen;
+	const unsigned char *iv;
+	size_t ivlen;
 };
 
 /**
  * AEAD data
+ * @assoc: Associated data - input
+ * @assoclen: Length of associated data - input
+ * @taglen: Length of authentication tag - input
+ * @tag: Authentication tag - input for decryption, output for encryption
+ * @retlen: internal data -- number of bytes returned by the read system call
  */
 struct kcapi_aead_data {
-	const unsigned char *assoc; /** <Associated data -- input */
-	size_t assoclen; /** <Length of associated data -- input */
-	size_t taglen; /** <Length of authentication tag -- input */
-	unsigned char *tag; /** <Authentication tag -- input for decryption,
-			     * output for encryption */
+	const unsigned char *assoc;
+	size_t assoclen;
+	size_t taglen;
+	unsigned char *tag;
+	size_t retlen;
 };
 
 /**
  * Cipher handle
+ * @tfmfd: Socket descriptor for AF_ALG
+ * @opfd: FD to open kernel crypto API TFM
+ * @skdata: Common data for all ciphers
+ * @aead: AEAD cipher specific data
  */
 struct kcapi_handle {
-	int tfmfd;	/** <Socket descriptor for AF_ALG */
-	int opfd;	/** <FD to open kernel crypto API TFM */
+	int tfmfd;
+	int opfd;
 	struct kcapi_skcipher_data skdata;
 	struct kcapi_aead_data aead;
 };
@@ -76,8 +91,8 @@ int kcapi_cipher_init(struct kcapi_handle *handle, const char *ciphername);
 int kcapi_cipher_destroy(struct kcapi_handle *handle);
 int kcapi_cipher_setkey(struct kcapi_handle *handle,
 			const unsigned char *key, size_t keylen);
-void kcapi_cipher_setiv(struct kcapi_handle *handle,
-			const unsigned char *iv, size_t ivlen);
+int kcapi_cipher_setiv(struct kcapi_handle *handle,
+		       const unsigned char *iv, size_t ivlen);
 ssize_t kcapi_cipher_encrypt(struct kcapi_handle *handle,
 			     const unsigned char *in, size_t inlen,
 			     unsigned char *out, size_t outlen);
@@ -91,19 +106,29 @@ int kcapi_aead_init(struct kcapi_handle *handle, const char *ciphername);
 int kcapi_aead_destroy(struct kcapi_handle *handle);
 int kcapi_aead_setkey(struct kcapi_handle *handle,
 		      const unsigned char *key, size_t keylen);
-void kcapi_aead_setiv(struct kcapi_handle *handle,
-		      const unsigned char *iv, size_t ivlen);
+int kcapi_aead_setiv(struct kcapi_handle *handle,
+		     const unsigned char *iv, size_t ivlen);
 void kcapi_aead_setassoc(struct kcapi_handle *handle,
 			 const unsigned char *assoc, size_t assoclen);
 void kcapi_aead_settaglen(struct kcapi_handle *handle, size_t taglen);
 ssize_t kcapi_aead_encrypt(struct kcapi_handle *handle,
 			   const unsigned char *in, size_t inlen,
 			   unsigned char *out, size_t outlen);
+ssize_t kcapi_aead_enc_nonalign(struct kcapi_handle *handle,
+				unsigned char *pt, size_t ptlen, size_t taglen);
+void kcapi_aead_enc_getdata(struct kcapi_handle *handle,
+			    unsigned char **ct, size_t *ctlen,
+			    unsigned char **tag, size_t *taglen);
+void kcapi_aead_enc_free(struct kcapi_handle *handle);
 ssize_t kcapi_aead_decrypt(struct kcapi_handle *handle,
 			   const unsigned char *in, size_t inlen,
 			   unsigned char *out, size_t outlen);
-void kcapi_aead_gettag(struct kcapi_handle *handle,
-		       unsigned char **tag, size_t *taglen);
+ssize_t kcapi_aead_dec_nonalign(struct kcapi_handle *handle,
+				unsigned char *ct, size_t ctlen,
+				unsigned char *tag, size_t taglen);
+void kcapi_aead_dec_getdata(struct kcapi_handle *handle,
+			    unsigned char **pt, size_t *ptlen);
+void kcapi_aead_dec_free(struct kcapi_handle *handle);
 int kcapi_aead_ivsize(struct kcapi_handle *handle);
 int kcapi_aead_blocksize(struct kcapi_handle *handle);
 int kcapi_aead_authsize(struct kcapi_handle *handle);
@@ -118,6 +143,7 @@ int kcapi_md_update(struct kcapi_handle *handle,
 		    const unsigned char *buffer, size_t len);
 ssize_t kcapi_md_final(struct kcapi_handle *handle,
 		       unsigned char *buffer, size_t len);
+int kcapi_md_digestsize(struct kcapi_handle *handle);
 
 int kcapi_rng_init(struct kcapi_handle *handle, const char *ciphername);
 int kcapi_rng_destroy(struct kcapi_handle *handle);
@@ -125,5 +151,8 @@ ssize_t kcapi_rng_generate(struct kcapi_handle *handle,
 			   unsigned char *buffer, size_t len);
 
 void kcapi_versionstring(char *buf, size_t buflen);
+int kcapi_pad_iv(struct kcapi_handle *handle,
+		 const unsigned char *iv, size_t ivlen,
+		 unsigned char **newiv, size_t *newivlen);
 
 #endif /* _KCAPI_H */
